;;; A little prime-tester for demo/benchmark purposes
;;; For C/C++ the same thumb algorithm was used


(proclaim '(inline primep))
(defun primep (x)
  "Predicate to test the primality of x."
  (cond
    ((not (integerp x)) nil)
    ((< x 2) nil)
    ((= x 2) 2)
    ((= 0 (mod x 2)) nil)
    (t (loop for i from 3 to (isqrt x) by 2
        never (= 0 (mod x i))))))


(defun nextprime (x)
  "Next prime greater x."
  (loop for i from x
    when (primep i) return i))


(defun findprimes (x n)
  "Find and print n prime numbers greater than x."
  (do ((i (1+ x) (1+ i)) (found 0))
    ((>= found n) t)
    (when (primep i)
      (format t "~A~%" i)
      (incf found))))


;; Approx. runtimes:
;; 0.20 sec C/C++ (base type ULL only)
;; 0.21 sec SBCL
;; 7,21 sec CLISP
(defun bench ()
  (time (findprimes 1000000000000 10)))
