#!/usr/local/bin/sbcl --script

;;; A little Lisp shell script. Choose your shebang e.g.
;;; #!/usr/local/bin/sbcl --script
;;; #!/usr/bin/clisp


;; Global for argument list
(defvar *args*)


;; Fetch command-line argument list
(defun fetcharglist ()
  "Tested for SBCL and CLISP only"
  (or 
    #+SBCL *posix-argv*
    #+CLISP (cons "clisp" ext:*args*)
    #+CMU extensions:*command-line-words*
    #+LISPWORKS system:*line-arguments-list*
    #+GCL si::*command-args*
    nil))


;; Rename files from to
(defun rename (from to)
  (dolist (file (directory from))
    (rename-file file (translate-pathname file from to))))


;; Rename files from to. Dry run with output info only
(defun rename-dry (from to)
  (dolist (file (directory from))
    (format t "FROM: ~A TO: ~A~%" file
            (translate-pathname file from to))))


;; Be generous and spend a help
(defun help ()
  (format t
    "~%~A~%~%"
    "Example usage: ./rename.sh \"*.lisp\" \"new-*.lisp\""))


;; For testing just show what WOULD happen
;; Filenames with wildcards MUST be quotet. See help above
(setq *args* (fetcharglist))
  (cond
    ((string= "-h" (nth 1 *args*)) (help))
    ((string= "--help" (nth 1 *args*)) (help))
    ((not (= (length *args*) 3)) (help))
    (t (rename-dry (nth 1 *args*) (nth 2 *args*))))
