/** *************************************************************************
 **
 ** Copyright (C) 2010 Heinz van Saanen
 **
 ** This file is part of the interactive editor CLedit.
 **
 ** CLedit may be used under the terms of the GNU General Public License
 ** version 3 or later as published by the Free Software Foundation and
 ** appearing in the file LICENSE.GPL included in this package.
 **
 ** CLedit is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
****************************************************************************/


#ifndef HIGH
#define HIGH


/** Includes **/
#include <QSyntaxHighlighter>
#include <QHash>
#include <QTextCharFormat>


/** Parenthesis extra stuff **/
struct ParenthesisInfo {
	char character;
	int position;
}; 


class TextBlockData : public QTextBlockUserData {

public:
	TextBlockData() {};
	QVector<ParenthesisInfo *> parentheses();
	void insert( ParenthesisInfo *info );

private:
	QVector<ParenthesisInfo *> m_parentheses;
};


/** Nomen est omen. Handle syntax highlighting **/
class Highlighter : public QSyntaxHighlighter {
	Q_OBJECT

public:
	Highlighter( QTextDocument *parent = 0 );

	// Add new/own highlighter rules here
	void loadLispRules();
	void removeRules() { m_highlightingRules.clear(); }
	QStringList wordList;

protected:
	void highlightBlock( const QString &text );

private:

	// Rule structure
	struct HighlightingRule {
		QRegExp pattern;
		QTextCharFormat format;
	};

	// Rule data
	HighlightingRule m_rule;
	QVector<HighlightingRule> m_highlightingRules;

	// Format data
	QTextCharFormat m_keywordFormat;
	QTextCharFormat m_commentFormat;
	QTextCharFormat m_quotationFormat;

	// Format data list for keywords/symbols
	QStringList m_keywordPatterns;
};

// HIGH
#endif
