/** *************************************************************************
 **
 ** Copyright (C) 2010 Heinz van Saanen
 **
 ** This file is part of the interactive editor CLedit.
 **
 ** CLedit may be used under the terms of the GNU General Public License
 ** version 3 or later as published by the Free Software Foundation and
 ** appearing in the file LICENSE.GPL included in this package.
 **
 ** CLedit is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
****************************************************************************/


#ifndef MAINWINDOW_H
#define MAINWINDOW_H


/** Includes **/
#include <QMainWindow>
#include <QCloseEvent>
#include <QTextBrowser>
#include <QStringList>
#include <QPlainTextEdit>
#include "ui_mainwindow.h"
#include "process.h"
#include "high.h"


/** MainWindow class **/
class MainWindow : public QMainWindow, Ui_MainWindow {
	Q_OBJECT

public:
	MainWindow( QWidget *parent = 0, Qt::WindowFlags f = 0 );

protected:
	// Re-implement closeEvent
	void closeEvent( QCloseEvent *event ) { event->accept(); exitSlot(); };

private:
	Process *m_process;					// Current process
	QString m_cmdStr;					// Holds prog to execute. Eg sbcl, clisp, crafty ...

	QString m_fileName;					// Last filename opened/saved 
	QPlainTextEdit *m_messageEdit;		// Message editor
	QPlainTextEdit *m_historyEdit;		// History editor
	QTextBrowser *m_helpEdit;			// Help editor

	Highlighter *m_highlighter;			// Editor highlighter stuff
	QCompleter *m_completer;			// CMD-line auto-completer
	void keyPressEvent( QKeyEvent * );	// Handle up/down keys for history 

	QStringList m_historyList;			// History list
	int m_historyCnt;					// Total entries
	int m_historyCur;					// Current entry

	void setupActions();				// Set up actions at program start

	// Parenthesis matcher
	bool matchLeftPar ( QTextBlock currentBlock, int index, int numRightPar );
	bool matchRightPar( QTextBlock currentBlock, int index, int numLeftPar );
	void createParSelection( int pos );

	// // Load/store settings
	void readSettings();
	void writeSettings();

private slots:

	// Menu "File"
	void newSlot();
	void openSlot();
	void save_asSlot();
	void printSlot();
	void exitSlot();

	// Menu "Edit"
	void execute_selectionSlot();
	void execute_allSlot();
	void history_upSlot();
	void history_downSlot();
	
	// Menu "Editors"
	void messagesSlot();		// Show messages editor
	void historySlot();			// Show history editor
	void show_AllSlot();		// Show all editors

	// Menu "Exe"
	void executableSlot();

	// Menu "Help"
	void helpSlot();
	void hyperspecSlot();
	void aboutSlot();

	// Process related
	void lineSlot();
	void lispHelpSlot();
	void completerSlot( const QString &text );
	void messageOutSlot( QString &mess );
	void procStateSlot ( bool b );

	// textEdit
	void contentsChangedSlot();

	// Parenthesis matcher
	void matchPar();
};

// MAINWINDOW_H
#endif
